-- raid icons
local RIB_RAIDICON_FILE = "Interface\\TargetingFrame\\UI-RaidTargetingIcons";
local RIB_RAIDFLARE_FILES = {};
RIB_RAIDFLARE_FILES[1] = "Interface\\AddOns\\RaidIconBar\\marker\\blue";
RIB_RAIDFLARE_FILES[2] = "Interface\\AddOns\\RaidIconBar\\marker\\green";
RIB_RAIDFLARE_FILES[3] = "Interface\\AddOns\\RaidIconBar\\marker\\purple";
RIB_RAIDFLARE_FILES[4] = "Interface\\AddOns\\RaidIconBar\\marker\\red";
RIB_RAIDFLARE_FILES[5] = "Interface\\AddOns\\RaidIconBar\\marker\\yellow";
RIB_RAIDFLARE_FILES[6] = "Interface\\AddOns\\RaidIconBar\\marker\\orange";
RIB_RAIDFLARE_FILES[7] = "Interface\\AddOns\\RaidIconBar\\marker\\silver";
RIB_RAIDFLARE_FILES[8] = "Interface\\AddOns\\RaidIconBar\\marker\\white";

local SCALE = 0.5;

local DEFAULT_SETTINGS = {
    ["IconBar"] = {
        ["point"] = "CENTER",
        ["relativeTo"] = nil,
        ["relativePoint"] = "BOTTOMLEFT",
        ["xOfs"] = 1000000,
        ["yOfs"] = 1000000
    },
    ["FlareBar"] = {
        ["point"] = "CENTER",
        ["relativeTo"] = nil,
        ["relativePoint"] = "BOTTOMLEFT",
        ["xOfs"] = 1000000,
        ["yOfs"] = 1000000
    },
};

function RIB_Init()
    RIB_RegisterEvents();
    RIB_InitSlashCommandHandler();

    EditModeManagerFrame:HookScript("OnShow", RIB_EditmodeButton_EnterEditMode);
    EditModeManagerFrame:HookScript("OnHide", RIB_EditmodeButton_ExitEditMode);
end

function RIB_EditmodeButton_EnterEditMode()
    RIB_ChatRaidIcons:SetMovable(true);
    RIB_ChatRaidFlares:SetMovable(true);
end

function RIB_EditmodeButton_ExitEditMode()
    RIB_ChatRaidIcons:SetMovable(false);
    RIB_ChatRaidFlares:SetMovable(false);
end

function RIB_RegisterEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");

    frame:HookScript("OnEvent", RIB_DataStoreOnEvent);
end

function RIB_DataStoreOnEvent(self, event, ...)
    if (event == "ADDON_LOADED" ) then
        if (... == "RaidIconBar") then
            RaidIconBarData = RaidIconBarData or DEFAULT_SETTINGS;

            createRaidIconBar();
            createRaidFlareBar();
        end
    end
end

function RIB_InitSlashCommandHandler()
    SLASH_RIB1 = "/rib";
    SLASH_RIB2 = "/raidiconbar";
    SlashCmdList["RIB"] = function(msg)
        RIB_SlashCommandHandler(msg);
    end
end

function RIB_SlashCommandHandler(msg)
    if (msg == "reload") then
        ReloadUI();
    elseif (msg == "show") then
        RIB_Fix();
    elseif (msg == "hide") then
        RIB_Reset();
    end
end

function RIB_Fix()
    RaidIconBarData = {
        ["IconBar"] = {
            ["point"] = "CENTER",
            ["relativeTo"] = nil,
            ["relativePoint"] = "BOTTOMLEFT",
            ["xOfs"] = 500,
            ["yOfs"] = 800
        },
        ["FlareBar"] = {
            ["point"] = "CENTER",
            ["relativeTo"] = nil,
            ["relativePoint"] = "BOTTOMLEFT",
            ["xOfs"] = 500,
            ["yOfs"] = 750
        },
    };

    RaidIconBar_ResetRaidIconPosition();
    RaidIconBar_ResetRaidFlarePosition();
end

function RIB_Reset()
    RaidIconBarData = DEFAULT_SETTINGS;
    
    RaidIconBar_ResetRaidIconPosition();
    RaidIconBar_ResetRaidFlarePosition();
end

------------------------------------------------------------------------------------------
---- RAID ICONS
------------------------------------------------------------------------------------------

function raidIconClicked(self, button, down)
    local index = self:GetAttribute("index");

    if (button == "LeftButton") then
        SetRaidTarget("target", index);
    elseif (button == "RightButton") then
        SetRaidTarget("target", 0);
    end
end

function createRaidIconBar()
    local raidicons = CreateFrame("Frame", "RIB_ChatRaidIcons", UIParent);
    raidicons:SetSize(130, 23);
    raidicons:SetScale(SCALE);
    raidicons:SetFrameStrata("DIALOG");
    raidicons:SetPoint(RaidIconBarData.IconBar.point, RaidIconBarData.IconBar.relativeTo or UIParent, RaidIconBarData.IconBar.relativePoint, RaidIconBarData.IconBar.xOfs, RaidIconBarData.IconBar.yOfs);

    for index=1, 8, 1 do
        local posx = (index - 5.5) * 50;

        local btn = CreateFrame("Button", "RIB_CHAT_RAIDFLARE_"..index, raidicons, "SecureActionButtonTemplate, ActionButtonTemplate");

        btn.NormalTexture:SetTexture(nil);
        btn.PushedTexture:SetTexture(nil);
		btn.SlotBackground:SetTexture(nil);

        btn.icon:SetTexture(RIB_RAIDICON_FILE);
        local left = mod((index-1)/4, 1);
        local right = left + 0.25;
        local top = floor((index-1)/4) * 0.25;
        local bottom = top + 0.25;
        btn.icon:SetTexCoord(left, right, top, bottom);

        btn:ClearAllPoints();
        btn:SetPoint("RIGHT", raidicons, "RIGHT", posx, 0);

        btn:SetAttribute("index", index);

        btn:RegisterForClicks("AnyUp");
        btn:HookScript("OnClick", raidIconClicked);

        btn:HookScript("OnMouseDown", RIB_RaidIcons_OnMouseDown);
        btn:HookScript("OnMouseUp", RIB_RaidIcons_OnMouseUp);
    end

    RaidIconBar_ResetRaidIconPosition();
end

function RIB_RaidIcons_OnMouseDown(frame, button)
    if (RIB_ChatRaidIcons:IsMovable()) then
        RIB_ChatRaidIcons:StartMoving();
    end
end

function RIB_RaidIcons_OnMouseUp(frame, button)
    if (RIB_ChatRaidIcons:IsMovable()) then
        RIB_ChatRaidIcons:StopMovingOrSizing();
        local point, relativeTo, relativePoint, xOfs, yOfs = RIB_ChatRaidIcons:GetPoint(1)
        RaidIconBarData.IconBar = {
            ["point"] = point,
            ["relativeTo"] = nil,
            ["relativePoint"] = relativePoint,
            ["xOfs"] = xOfs,
            ["yOfs"] = yOfs
        }
    end
end

function RaidIconBar_RaidIconSnap()
    if (InCombatLockdown()) then
        return;
    end

    if (_G["RIB_ChatRaidIcons"] == nil) then
        createRaidIconBar();
    end

    local frame = _G["RIB_ChatRaidIcons"];

    icons_posx = frame:GetLeft();
    icons_posy = frame:GetBottom();

    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();

    x = x / scale / SCALE;
    y = y / scale / SCALE;

    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
end

function RaidIconBar_ResetRaidIconPosition()
    local frame = _G["RIB_ChatRaidIcons"];

    if (frame) then
        frame:ClearAllPoints();
        frame:SetPoint(RaidIconBarData.IconBar.point, RaidIconBarData.IconBar.relativeTo or UIParent, RaidIconBarData.IconBar.relativePoint, RaidIconBarData.IconBar.xOfs, RaidIconBarData.IconBar.yOfs);
    end
end

------------------------------------------------------------------------------------------
---- RAID FLARES
------------------------------------------------------------------------------------------

function createRaidFlareBar()
    local raidflares = CreateFrame("Frame", "RIB_ChatRaidFlares", UIParent);
    raidflares:SetSize(130, 23);
    raidflares:SetScale(SCALE);
    raidflares:SetFrameStrata("DIALOG");
    raidflares:SetPoint(RaidIconBarData.FlareBar.point, RaidIconBarData.FlareBar.relativeTo or UIParent, RaidIconBarData.FlareBar.relativePoint, RaidIconBarData.FlareBar.xOfs, RaidIconBarData.FlareBar.yOfs);

    for index=1, 8, 1 do
        local posx = (index - 5.5) * 50;

        local btn = CreateFrame("Button", "RIB_CHAT_RAIDFLARE_"..index, raidflares, "SecureActionButtonTemplate, ActionButtonTemplate");

        btn.NormalTexture:SetTexture(nil);
        btn.PushedTexture:SetTexture(nil);
		btn.SlotBackground:SetTexture(nil);

        btn.icon:SetTexture(RIB_RAIDFLARE_FILES[index]);

        btn:RegisterForClicks("AnyUp");
        btn:SetAttribute("type1", "macro");
        btn:SetAttribute("macrotext1", "/worldmarker "..index);
        btn:SetAttribute("type2", "macro");
        btn:SetAttribute("macrotext2", "/clearworldmarker "..index);

        btn:ClearAllPoints();
        btn:SetPoint("RIGHT", raidflares, "RIGHT", posx, 0);

        btn:SetAttribute("index", index);

        btn:HookScript("OnMouseDown", RIB_RaidFlares_OnMouseDown);
        btn:HookScript("OnMouseUp", RIB_RaidFlares_OnMouseUp);
    end

    RaidIconBar_ResetRaidFlarePosition();
end

function RIB_RaidFlares_OnMouseDown(frame, button)
    if (RIB_ChatRaidFlares:IsMovable()) then
        RIB_ChatRaidFlares:StartMoving();
    end
end

function RIB_RaidFlares_OnMouseUp(frame, button)
    if (RIB_ChatRaidFlares:IsMovable()) then
        RIB_ChatRaidFlares:StopMovingOrSizing();
        local point, relativeTo, relativePoint, xOfs, yOfs = RIB_ChatRaidFlares:GetPoint(1)
        RaidIconBarData.FlareBar = {
            ["point"] = point,
            ["relativeTo"] = nil,
            ["relativePoint"] = relativePoint,
            ["xOfs"] = xOfs,
            ["yOfs"] = yOfs
        }
    end
end

function RaidIconBar_RaidFlareSnap()
    if (InCombatLockdown()) then
        return;
    end

    if (_G["RIB_ChatRaidFlares"] == nil) then
        createRaidFlareBar();
    end

    local frame = _G["RIB_ChatRaidFlares"];

    icons_posx = frame:GetLeft();
    icons_posy = frame:GetBottom();

    local x, y = GetCursorPosition();
    local scale = UIParent:GetEffectiveScale();

    x = x / scale / SCALE;
    y = y / scale / SCALE;

    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
end

function RaidIconBar_ResetRaidFlarePosition()
    local frame = _G["RIB_ChatRaidFlares"];

    if (frame) then
        frame:ClearAllPoints();
        frame:SetPoint(RaidIconBarData.FlareBar.point, RaidIconBarData.FlareBar.relativeTo or UIParent, RaidIconBarData.FlareBar.relativePoint, RaidIconBarData.FlareBar.xOfs, RaidIconBarData.FlareBar.yOfs);
    end
end

RIB_Init();
